@@ -251,6 +251,16 @@ static void champlain_view_go_to_with_duration (ChamplainView *view,
     guint duration);
 static gboolean redraw_timeout_cb(gpointer view);
 static void remove_all_tiles (ChamplainView *view);
+static void get_x_y_for_zoom_level (ChamplainView *view,
+    guint zoom_level,
+    gint offset_x,
+    gint offset_y,
+    gdouble *new_x,
+    gdouble *new_y);
+static ChamplainBoundingBox *get_bounding_box (ChamplainView *view,
+    guint zoom_level,
+    gdouble x,
+    gdouble y);
 
 
 static void
@@ -2185,6 +2195,8 @@ champlain_view_set_map_source (ChamplainView *view,
       CHAMPLAIN_IS_MAP_SOURCE (source));
 
   ChamplainViewPrivate *priv = view->priv;
+  guint source_min_zoom;
+  guint source_max_zoom;
 
   if (priv->map_source == source)
     return;
@@ -2195,8 +2207,10 @@ champlain_view_set_map_source (ChamplainView *view,
   g_list_free_full (priv->overlay_sources, g_object_unref);
   priv->overlay_sources = NULL;
 
-  priv->min_zoom_level = champlain_map_source_get_min_zoom_level (priv->map_source);
-  priv->max_zoom_level = champlain_map_source_get_max_zoom_level (priv->map_source);
+  source_min_zoom = champlain_map_source_get_min_zoom_level (priv->map_source);
+  champlain_view_set_min_zoom_level (view, source_min_zoom);
+  source_max_zoom = champlain_map_source_get_max_zoom_level (priv->map_source);
+  champlain_view_set_max_zoom_level (view, source_max_zoom);
 
   /* Keep same zoom level if the new map supports it */
   if (priv->zoom_level > priv->max_zoom_level)
@@ -2632,6 +2646,22 @@ show_zoom_actor (ChamplainView *view,
     clutter_actor_set_scale (zoom_actor, deltazoom, deltazoom);
 }
 
+static void
+get_x_y_for_zoom_level (ChamplainView *view,
+                        guint zoom_level,
+                        gint offset_x,
+                        gint offset_y,
+                        gdouble *new_x,
+                        gdouble *new_y)
+{
+  ChamplainViewPrivate *priv = view->priv;
+  gdouble deltazoom;
+
+  deltazoom = pow (2, (gdouble) zoom_level - (gdouble) priv->zoom_level);
+
+  *new_x = (priv->viewport_x + offset_x) * deltazoom - offset_x;
+  *new_y = (priv->viewport_y + offset_y) * deltazoom - offset_y;
+}
 
 /* Sets the zoom level, leaving the (x, y) at the exact same point in the view */
 static gboolean
@@ -2647,7 +2677,6 @@ view_set_zoom_level_at (ChamplainView *view,
   gdouble new_x, new_y;
   gdouble offset_x = x;
   gdouble offset_y = y;
-  gdouble deltazoom;
 
   if (zoom_level == priv->zoom_level || ZOOM_LEVEL_OUT_OF_RANGE (priv, zoom_level))
     return FALSE;
@@ -2664,11 +2693,8 @@ view_set_zoom_level_at (ChamplainView *view,
   if (CLUTTER_ACTOR_IS_REALIZED (view))
     show_zoom_actor (view, zoom_level, offset_x, offset_y);
 
-  deltazoom = pow (2, (gdouble)zoom_level - (gdouble)priv->zoom_level);
+  get_x_y_for_zoom_level (view, zoom_level, offset_x, offset_y, &new_x, &new_y);
 
-  new_x = (priv->viewport_x + offset_x) * deltazoom - offset_x;
-  new_y = (priv->viewport_y + offset_y) * deltazoom - offset_y;
-  
   priv->zoom_level = zoom_level;
 
   if (CLUTTER_ACTOR_IS_REALIZED (view))
@@ -3017,6 +3043,61 @@ champlain_view_get_state (ChamplainView *view)
   return view->priv->state;
 }
 
+static ChamplainBoundingBox *
+get_bounding_box (ChamplainView *view,
+                  guint zoom_level,
+                  gdouble x,
+                  gdouble y)
+{
+  ChamplainViewPrivate *priv = view->priv;
+  ChamplainBoundingBox *bbox;
+
+  bbox = champlain_bounding_box_new ();
+
+  bbox->top = champlain_map_source_get_latitude (priv->map_source,
+                                                 zoom_level,
+                                                 y);
+  bbox->bottom = champlain_map_source_get_latitude (priv->map_source,
+                                                    zoom_level,
+                                                    y + priv->viewport_height);
+  bbox->left = champlain_map_source_get_longitude (priv->map_source,
+                                                   zoom_level,
+                                                   x);
+  bbox->right = champlain_map_source_get_longitude (priv->map_source,
+                                                    zoom_level,
+                                                    x + priv->viewport_width);
+  return bbox;
+}
+
+/**
+ * champlain_view_get_bounding_box_for_zoom_level:
+ * @view: a #ChamplainView
+ * @zoom_level: the level of zoom, a guint between 1 and 20
+ *
+ * Gets the bounding box for view @view at @zoom_level.
+ *
+ * Returns: (transfer full): the bounding box for the view at @zoom_level.
+ *
+ * Since: 0.12.7
+ */
+ChamplainBoundingBox *
+champlain_view_get_bounding_box_for_zoom_level (ChamplainView *view,
+                                                guint zoom_level)
+{
+  ChamplainViewPrivate *priv = view->priv;
+  gdouble x, y;
+  gdouble offset_x, offset_y;
+
+  g_return_val_if_fail (CHAMPLAIN_IS_VIEW (view), NULL);
+
+  offset_x = priv->viewport_width / 2.0;
+  offset_y = priv->viewport_height / 2.0;
+
+  get_x_y_for_zoom_level (view, zoom_level, offset_x, offset_y, &x, &y);
+
+  return get_bounding_box (view, zoom_level, x, y);
+}
+
 /**
  * champlain_view_get_bounding_box:
  * @view: a #ChamplainView
@@ -3033,34 +3114,17 @@ champlain_view_get_bounding_box (ChamplainView *view)
   DEBUG_LOG ()
 
   ChamplainViewPrivate *priv = view->priv;
-  ChamplainBoundingBox *bbox;
 
   g_return_val_if_fail (CHAMPLAIN_IS_VIEW (view), NULL);
 
-  bbox = champlain_bounding_box_new ();
-
-  bbox->top = champlain_map_source_get_latitude (priv->map_source,
-    priv->zoom_level,
-    priv->viewport_y);
-  bbox->bottom = champlain_map_source_get_latitude (priv->map_source,
-    priv->zoom_level,
-    priv->viewport_y + priv->viewport_height);
-
-  bbox->left = champlain_map_source_get_longitude (priv->map_source,
-    priv->zoom_level,
-    priv->viewport_x);
-  bbox->right = champlain_map_source_get_longitude (priv->map_source,
-    priv->zoom_level,
-    priv->viewport_x + priv->viewport_width);
-
-  return bbox;
+  return get_bounding_box (view, priv->zoom_level, priv->viewport_x, priv->viewport_y);
 }
 
 
 /**
  * champlain_view_add_overlay_source:
  * @view: a #ChamplainView
- * @source: a #ChamplainMapSource
+ * @map_source: a #ChamplainMapSource
  * @opacity: opacity to use
  *
  * Adds a new overlay map source to render tiles with the supplied opacity on top 
@@ -3070,7 +3134,7 @@ champlain_view_get_bounding_box (ChamplainView *view)
  */
 void
 champlain_view_add_overlay_source (ChamplainView *view,
-    ChamplainMapSource *source,
+    ChamplainMapSource *map_source,
     guint8 opacity)
 {
   DEBUG_LOG ()
@@ -3078,12 +3142,12 @@ champlain_view_add_overlay_source (ChamplainView *view,
   ChamplainViewPrivate *priv;
 
   g_return_if_fail (CHAMPLAIN_IS_VIEW (view));
-  g_return_if_fail (CHAMPLAIN_IS_MAP_SOURCE (source));
+  g_return_if_fail (CHAMPLAIN_IS_MAP_SOURCE (map_source));
 
   priv = view->priv;
-  g_object_ref (source);
-  priv->overlay_sources = g_list_append (priv->overlay_sources, source);
-  g_object_set_data (G_OBJECT (source), "opacity", GINT_TO_POINTER (opacity));
+  g_object_ref (map_source);
+  priv->overlay_sources = g_list_append (priv->overlay_sources, map_source);
+  g_object_set_data (G_OBJECT (map_source), "opacity", GINT_TO_POINTER (opacity));
   g_object_notify (G_OBJECT (view), "map-source");
 
   champlain_view_reload_tiles (view);
@@ -3093,7 +3157,7 @@ champlain_view_add_overlay_source (ChamplainView *view,
 /**
  * champlain_view_remove_overlay_source:
  * @view: a #ChamplainView
- * @source: a #ChamplainMapSource
+ * @map_source: a #ChamplainMapSource
  *
  * Removes an overlay source from #ChamplainView.
  *
@@ -3101,18 +3165,18 @@ champlain_view_add_overlay_source (ChamplainView *view,
  */
 void
 champlain_view_remove_overlay_source (ChamplainView *view,
-    ChamplainMapSource *source)
+    ChamplainMapSource *map_source)
 {
   DEBUG_LOG ()
 
   ChamplainViewPrivate *priv;
 
   g_return_if_fail (CHAMPLAIN_IS_VIEW (view));
-  g_return_if_fail (CHAMPLAIN_IS_MAP_SOURCE (source));
+  g_return_if_fail (CHAMPLAIN_IS_MAP_SOURCE (map_source));
 
   priv = view->priv;
-  priv->overlay_sources = g_list_remove (priv->overlay_sources, source);
-  g_object_unref (source);
+  priv->overlay_sources = g_list_remove (priv->overlay_sources, map_source);
+  g_object_unref (map_source);
   g_object_notify (G_OBJECT (view), "map-source");
 
   champlain_view_reload_tiles (view);
